@@ -16,16 +16,9 @@
 # permissions and limitations under the License.
 
 """Unittest for textfsm module."""
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
 
-from builtins import str
+import io
 import unittest
-from io import StringIO
-
-
 
 import textfsm
 
@@ -60,23 +53,45 @@ class UnitTestFSM(unittest.TestCase):
     self.assertEqual(v.OptionNames(), ['Required'])
 
     # regex must be bounded by parenthesis.
-    self.assertRaises(textfsm.TextFSMTemplateError,
-                      v.Parse,
-                      'Value beer (boo(hoo)))boo')
-    self.assertRaises(textfsm.TextFSMTemplateError,
-                      v.Parse,
-                      'Value beer boo(boo(hoo)))')
-    self.assertRaises(textfsm.TextFSMTemplateError,
-                      v.Parse,
-                      'Value beer (boo)hoo)')
+    self.assertRaises(
+        textfsm.TextFSMTemplateError, v.Parse, 'Value beer (boo(hoo)))boo'
+    )
+    self.assertRaises(
+        textfsm.TextFSMTemplateError, v.Parse, 'Value beer boo(boo(hoo)))'
+    )
+    self.assertRaises(
+        textfsm.TextFSMTemplateError, v.Parse, 'Value beer (boo)hoo)'
+    )
+
+    # Escaped parentheses don't count.
+    v = textfsm.TextFSMValue(options_class=textfsm.TextFSMOptions)
+    v.Parse(r'Value beer (boo\)hoo)')
+    self.assertEqual(v.name, 'beer')
+    self.assertEqual(v.regex, r'(boo\)hoo)')
+    self.assertRaises(
+        textfsm.TextFSMTemplateError, v.Parse, r'Value beer (boohoo\)'
+    )
+    self.assertRaises(
+        textfsm.TextFSMTemplateError, v.Parse, r'Value beer (boo)hoo\)'
+    )
+
+    # Unbalanced parenthesis can exist if within square "[]" braces.
+    v = textfsm.TextFSMValue(options_class=textfsm.TextFSMOptions)
+    v.Parse('Value beer (boo[(]hoo)')
+    self.assertEqual(v.name, 'beer')
+    self.assertEqual(v.regex, '(boo[(]hoo)')
+
+    # Escaped braces don't count.
+    self.assertRaises(
+        textfsm.TextFSMTemplateError, v.Parse, r'Value beer (boo\[)\]hoo)'
+    )
 
     # String function.
     v = textfsm.TextFSMValue(options_class=textfsm.TextFSMOptions)
     v.Parse('Value Required beer (boo(hoo))')
     self.assertEqual(str(v), 'Value Required beer (boo(hoo))')
     v = textfsm.TextFSMValue(options_class=textfsm.TextFSMOptions)
-    v.Parse(
-        r'Value Required,Filldown beer (bo\S+(hoo))')
+    v.Parse(r'Value Required,Filldown beer (bo\S+(hoo))')
     self.assertEqual(str(v), r'Value Required,Filldown beer (bo\S+(hoo))')
 
   def testFSMRule(self):
@@ -121,144 +136,174 @@ class UnitTestFSM(unittest.TestCase):
     self.assertEqual(r.record_op, 'NoRecord')
 
     # Bad syntax tests.
-    self.assertRaises(textfsm.TextFSMTemplateError, textfsm.TextFSMRule,
-                      '  ^A beer called ${beer} -> Next Next Next')
-    self.assertRaises(textfsm.TextFSMTemplateError, textfsm.TextFSMRule,
-                      '  ^A beer called ${beer} -> Boo.hoo')
-    self.assertRaises(textfsm.TextFSMTemplateError, textfsm.TextFSMRule,
-                      '  ^A beer called ${beer} -> Continue.Record $Hi')
+    self.assertRaises(
+        textfsm.TextFSMTemplateError,
+        textfsm.TextFSMRule,
+        '  ^A beer called ${beer} -> Next Next Next',
+    )
+    self.assertRaises(
+        textfsm.TextFSMTemplateError,
+        textfsm.TextFSMRule,
+        '  ^A beer called ${beer} -> Boo.hoo',
+    )
+    self.assertRaises(
+        textfsm.TextFSMTemplateError,
+        textfsm.TextFSMRule,
+        '  ^A beer called ${beer} -> Continue.Record $Hi',
+    )
 
   def testRulePrefixes(self):
     """Test valid and invalid rule prefixes."""
 
     # Bad syntax tests.
     for prefix in (' ', '.^', ' \t', ''):
-      f = StringIO('Value unused (.)\n\nStart\n' + prefix + 'A simple string.')
+      f = io.StringIO(
+          'Value unused (.)\n\nStart\n' + prefix + 'A simple string.'
+      )
       self.assertRaises(textfsm.TextFSMTemplateError, textfsm.TextFSM, f)
 
     # Good syntax tests.
     for prefix in (' ^', '  ^', '\t^'):
-      f = StringIO('Value unused (.)\n\nStart\n' + prefix + 'A simple string.')
+      f = io.StringIO(
+          'Value unused (.)\n\nStart\n' + prefix + 'A simple string.'
+      )
       self.assertIsNotNone(textfsm.TextFSM(f))
 
   def testImplicitDefaultRules(self):
 
-    for line in ('  ^A beer called ${beer} -> Record End',
-                 '  ^A beer called ${beer} -> End',
-                 '  ^A beer called ${beer} -> Next.NoRecord End',
-                 '  ^A beer called ${beer} -> Clear End',
-                 '  ^A beer called ${beer} -> Error "Hello World"'):
+    for line in (
+        '  ^A beer called ${beer} -> Record End',
+        '  ^A beer called ${beer} -> End',
+        '  ^A beer called ${beer} -> Next.NoRecord End',
+        '  ^A beer called ${beer} -> Clear End',
+        '  ^A beer called ${beer} -> Error "Hello World"',
+    ):
       r = textfsm.TextFSMRule(line)
       self.assertEqual(str(r), line)
 
-    for line in ('  ^A beer called ${beer} -> Next "Hello World"',
-                 '  ^A beer called ${beer} -> Record.Next',
-                 '  ^A beer called ${beer} -> Continue End',
-                 '  ^A beer called ${beer} -> Beer End'):
-      self.assertRaises(textfsm.TextFSMTemplateError,
-                        textfsm.TextFSMRule, line)
+    for line in (
+        '  ^A beer called ${beer} -> Next "Hello World"',
+        '  ^A beer called ${beer} -> Record.Next',
+        '  ^A beer called ${beer} -> Continue End',
+        '  ^A beer called ${beer} -> Beer End',
+    ):
+      self.assertRaises(textfsm.TextFSMTemplateError, textfsm.TextFSMRule, line)
 
   def testSpacesAroundAction(self):
-    for line in ('  ^Hello World -> Boo',
-                 '  ^Hello World ->  Boo',
-                 '  ^Hello World ->   Boo'):
-      self.assertEqual(
-          str(textfsm.TextFSMRule(line)), '  ^Hello World -> Boo')
+    for line in (
+        '  ^Hello World -> Boo',
+        '  ^Hello World ->  Boo',
+        '  ^Hello World ->   Boo',
+    ):
+      self.assertEqual(str(textfsm.TextFSMRule(line)), '  ^Hello World -> Boo')
 
     # A '->' without a leading space is considered part of the matching line.
-    self.assertEqual('  A simple line-> Boo -> Next',
-                     str(textfsm.TextFSMRule('  A simple line-> Boo -> Next')))
+    self.assertEqual(
+        '  A simple line-> Boo -> Next',
+        str(textfsm.TextFSMRule('  A simple line-> Boo -> Next')),
+    )
 
   def testParseFSMVariables(self):
     # Trivial template to initiate object.
-    f = StringIO('Value unused (.)\n\nStart\n')
+    f = io.StringIO('Value unused (.)\n\nStart\n')
     t = textfsm.TextFSM(f)
 
     # Trivial entry
     buf = 'Value Filldown Beer (beer)\n\n'
-    f = StringIO(buf)
+    f = io.StringIO(buf)
     t._ParseFSMVariables(f)
 
     # Single variable with commented header.
     buf = '# Headline\nValue Filldown Beer (beer)\n\n'
-    f = StringIO(buf)
+    f = io.StringIO(buf)
     t._ParseFSMVariables(f)
     self.assertEqual(str(t._GetValue('Beer')), 'Value Filldown Beer (beer)')
 
     # Multiple variables.
-    buf = ('# Headline\n'
-           'Value Filldown Beer (beer)\n'
-           'Value Required Spirits (whiskey)\n'
-           'Value Filldown Wine (claret)\n'
-           '\n')
+    buf = (
+        '# Headline\n'
+        'Value Filldown Beer (beer)\n'
+        'Value Required Spirits (whiskey)\n'
+        'Value Filldown Wine (claret)\n'
+        '\n'
+    )
     t._line_num = 0
-    f = StringIO(buf)
+    f = io.StringIO(buf)
     t._ParseFSMVariables(f)
     self.assertEqual(str(t._GetValue('Beer')), 'Value Filldown Beer (beer)')
     self.assertEqual(
-        str(t._GetValue('Spirits')), 'Value Required Spirits (whiskey)')
+        str(t._GetValue('Spirits')), 'Value Required Spirits (whiskey)'
+    )
     self.assertEqual(str(t._GetValue('Wine')), 'Value Filldown Wine (claret)')
 
     # Multiple variables.
-    buf = ('# Headline\n'
-           'Value Filldown Beer (beer)\n'
-           ' # A comment\n'
-           'Value Spirits ()\n'
-           'Value Filldown,Required Wine ((c|C)laret)\n'
-           '\n')
+    buf = (
+        '# Headline\n'
+        'Value Filldown Beer (beer)\n'
+        ' # A comment\n'
+        'Value Spirits ()\n'
+        'Value Filldown,Required Wine ((c|C)laret)\n'
+        '\n'
+    )
 
-    f = StringIO(buf)
+    f = io.StringIO(buf)
     t._ParseFSMVariables(f)
     self.assertEqual(str(t._GetValue('Beer')), 'Value Filldown Beer (beer)')
+    self.assertEqual(str(t._GetValue('Spirits')), 'Value Spirits ()')
     self.assertEqual(
-        str(t._GetValue('Spirits')), 'Value Spirits ()')
-    self.assertEqual(str(t._GetValue('Wine')),
-                     'Value Filldown,Required Wine ((c|C)laret)')
+        str(t._GetValue('Wine')), 'Value Filldown,Required Wine ((c|C)laret)'
+    )
 
     # Malformed variables.
     buf = 'Value Beer (beer) beer'
-    f = StringIO(buf)
+    f = io.StringIO(buf)
     self.assertRaises(textfsm.TextFSMTemplateError, t._ParseFSMVariables, f)
 
     buf = 'Value Filldown, Required Spirits ()'
-    f = StringIO(buf)
+    f = io.StringIO(buf)
     self.assertRaises(textfsm.TextFSMTemplateError, t._ParseFSMVariables, f)
     buf = 'Value filldown,Required Wine ((c|C)laret)'
-    f = StringIO(buf)
+    f = io.StringIO(buf)
     self.assertRaises(textfsm.TextFSMTemplateError, t._ParseFSMVariables, f)
 
     # Values that look bad but are OK.
-    buf = ('# Headline\n'
-           'Value Filldown Beer (bee(r), (and) (M)ead$)\n'
-           '# A comment\n'
-           'Value Spirits,and,some ()\n'
-           'Value Filldown,Required Wine ((c|C)laret)\n'
-           '\n')
-    f = StringIO(buf)
+    buf = (
+        '# Headline\n'
+        'Value Filldown Beer (bee(r), (and) (M)ead$)\n'
+        '# A comment\n'
+        'Value Spirits,and,some ()\n'
+        'Value Filldown,Required Wine ((c|C)laret)\n'
+        '\n'
+    )
+    f = io.StringIO(buf)
     t._ParseFSMVariables(f)
-    self.assertEqual(str(t._GetValue('Beer')),
-                     'Value Filldown Beer (bee(r), (and) (M)ead$)')
     self.assertEqual(
-        str(t._GetValue('Spirits,and,some')), 'Value Spirits,and,some ()')
-    self.assertEqual(str(t._GetValue('Wine')),
-                     'Value Filldown,Required Wine ((c|C)laret)')
+        str(t._GetValue('Beer')), 'Value Filldown Beer (bee(r), (and) (M)ead$)'
+    )
+    self.assertEqual(
+        str(t._GetValue('Spirits,and,some')), 'Value Spirits,and,some ()'
+    )
+    self.assertEqual(
+        str(t._GetValue('Wine')), 'Value Filldown,Required Wine ((c|C)laret)'
+    )
 
     # Variable name too long.
-    buf = ('Value Filldown '
-           'nametoolong_nametoolong_nametoolo_nametoolong_nametoolong '
-           '(beer)\n\n')
-    f = StringIO(buf)
-    self.assertRaises(textfsm.TextFSMTemplateError,
-                      t._ParseFSMVariables, f)
+    buf = (
+        'Value Filldown '
+        'nametoolong_nametoolong_nametoolo_nametoolong_nametoolong '
+        '(beer)\n\n'
+    )
+    f = io.StringIO(buf)
+    self.assertRaises(textfsm.TextFSMTemplateError, t._ParseFSMVariables, f)
 
   def testParseFSMState(self):
 
-    f = StringIO('Value Beer (.)\nValue Wine (\\w)\n\nStart\n')
+    f = io.StringIO('Value Beer (.)\nValue Wine (\\w)\n\nStart\n')
     t = textfsm.TextFSM(f)
 
     # Fails as we already have 'Start' state.
     buf = 'Start\n  ^.\n'
-    f = StringIO(buf)
+    f = io.StringIO(buf)
     self.assertRaises(textfsm.TextFSMTemplateError, t._ParseFSMState, f)
 
     # Remove start so we can test new Start state.
@@ -266,7 +311,7 @@ class UnitTestFSM(unittest.TestCase):
 
     # Single state.
     buf = '# Headline\nStart\n  ^.\n\n'
-    f = StringIO(buf)
+    f = io.StringIO(buf)
     t._ParseFSMState(f)
     self.assertEqual(str(t.states['Start'][0]), '  ^.')
     try:
@@ -276,7 +321,7 @@ class UnitTestFSM(unittest.TestCase):
 
     # Multiple states.
     buf = '# Headline\nStart\n  ^.\n  ^Hello World\n  ^Last-[Cc]ha$$nge\n'
-    f = StringIO(buf)
+    f = io.StringIO(buf)
     t._line_num = 0
     t.states = {}
     t._ParseFSMState(f)
@@ -292,21 +337,23 @@ class UnitTestFSM(unittest.TestCase):
     t.states = {}
     # Malformed states.
     buf = 'St%art\n  ^.\n  ^Hello World\n'
-    f = StringIO(buf)
+    f = io.StringIO(buf)
     self.assertRaises(textfsm.TextFSMTemplateError, t._ParseFSMState, f)
 
     buf = 'Start\n^.\n  ^Hello World\n'
-    f = StringIO(buf)
+    f = io.StringIO(buf)
     self.assertRaises(textfsm.TextFSMTemplateError, t._ParseFSMState, f)
 
     buf = '  Start\n  ^.\n  ^Hello World\n'
-    f = StringIO(buf)
+    f = io.StringIO(buf)
     self.assertRaises(textfsm.TextFSMTemplateError, t._ParseFSMState, f)
 
     # Multiple variables and substitution (depends on _ParseFSMVariables).
-    buf = ('# Headline\nStart\n  ^.${Beer}${Wine}.\n'
-           '  ^Hello $Beer\n  ^Last-[Cc]ha$$nge\n')
-    f = StringIO(buf)
+    buf = (
+        '# Headline\nStart\n  ^.${Beer}${Wine}.\n'
+        '  ^Hello $Beer\n  ^Last-[Cc]ha$$nge\n'
+    )
+    f = io.StringIO(buf)
     t.states = {}
     t._ParseFSMState(f)
     self.assertEqual(str(t.states['Start'][0]), '  ^.${Beer}${Wine}.')
@@ -321,43 +368,52 @@ class UnitTestFSM(unittest.TestCase):
 
     # State name too long (>32 char).
     buf = 'rnametoolong_nametoolong_nametoolong_nametoolong_nametoolo\n  ^.\n\n'
-    f = StringIO(buf)
+    f = io.StringIO(buf)
     self.assertRaises(textfsm.TextFSMTemplateError, t._ParseFSMState, f)
 
   def testInvalidStates(self):
 
     # 'Continue' should not accept a destination.
-    self.assertRaises(textfsm.TextFSMTemplateError, textfsm.TextFSMRule,
-                      '^.* -> Continue Start')
+    self.assertRaises(
+        textfsm.TextFSMTemplateError,
+        textfsm.TextFSMRule,
+        '^.* -> Continue Start',
+    )
 
     # 'Error' accepts a text string but "next' state does not.
-    self.assertEqual(str(textfsm.TextFSMRule('  ^ -> Error "hi there"')),
-                     '  ^ -> Error "hi there"')
-    self.assertRaises(textfsm.TextFSMTemplateError, textfsm.TextFSMRule,
-                      '^.* -> Next "Hello World"')
+    self.assertEqual(
+        str(textfsm.TextFSMRule('  ^ -> Error "hi there"')),
+        '  ^ -> Error "hi there"',
+    )
+    self.assertRaises(
+        textfsm.TextFSMTemplateError,
+        textfsm.TextFSMRule,
+        '^.* -> Next "Hello World"',
+    )
 
   def testRuleStartsWithCarrot(self):
 
-    f = StringIO(
-        'Value Beer (.)\nValue Wine (\\w)\n\nStart\n  A Simple line')
+    f = io.StringIO(
+        'Value Beer (.)\nValue Wine (\\w)\n\nStart\n  A Simple line'
+    )
     self.assertRaises(textfsm.TextFSMTemplateError, textfsm.TextFSM, f)
 
   def testValidateFSM(self):
 
     # No Values.
-    f = StringIO('\nNotStart\n')
+    f = io.StringIO('\nNotStart\n')
     self.assertRaises(textfsm.TextFSMTemplateError, textfsm.TextFSM, f)
 
     # No states.
-    f = StringIO('Value unused (.)\n\n')
+    f = io.StringIO('Value unused (.)\n\n')
     self.assertRaises(textfsm.TextFSMTemplateError, textfsm.TextFSM, f)
 
     # No 'Start' state.
-    f = StringIO('Value unused (.)\n\nNotStart\n')
+    f = io.StringIO('Value unused (.)\n\nNotStart\n')
     self.assertRaises(textfsm.TextFSMTemplateError, textfsm.TextFSM, f)
 
     # Has 'Start' state with valid destination
-    f = StringIO('Value unused (.)\n\nStart\n')
+    f = io.StringIO('Value unused (.)\n\nStart\n')
     t = textfsm.TextFSM(f)
     t.states['Start'] = []
     t.states['Start'].append(textfsm.TextFSMRule('^.* -> Start'))
@@ -389,14 +445,14 @@ class UnitTestFSM(unittest.TestCase):
     # Trivial template
     buf = 'Value Beer (.*)\n\nStart\n  ^\\w\n'
     buf_result = buf
-    f = StringIO(buf)
+    f = io.StringIO(buf)
     t = textfsm.TextFSM(f)
     self.assertEqual(str(t), buf_result)
 
     # Slightly more complex, multple vars.
     buf = 'Value A (.*)\nValue B (.*)\n\nStart\n  ^\\w\n\nState1\n  ^.\n'
     buf_result = buf
-    f = StringIO(buf)
+    f = io.StringIO(buf)
     t = textfsm.TextFSM(f)
     self.assertEqual(str(t), buf_result)
 
@@ -404,7 +460,7 @@ class UnitTestFSM(unittest.TestCase):
 
     # Trivial FSM, no records produced.
     tplt = 'Value unused (.)\n\nStart\n  ^Trivial SFM\n'
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
 
     data = 'Non-matching text\nline1\nline 2\n'
     self.assertFalse(t.ParseText(data))
@@ -414,7 +470,7 @@ class UnitTestFSM(unittest.TestCase):
 
     # Simple FSM, One Variable no options.
     tplt = 'Value boo (.*)\n\nStart\n  ^$boo -> Next.Record\n\nEOF\n'
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
 
     # Matching one line.
     # Tests 'Next' & 'Record' actions.
@@ -429,10 +485,12 @@ class UnitTestFSM(unittest.TestCase):
     self.assertListEqual(result, [['Matching text'], ['And again']])
 
     # Two Variables and singular options.
-    tplt = ('Value Required boo (one)\nValue Filldown hoo (two)\n\n'
-            'Start\n  ^$boo -> Next.Record\n  ^$hoo -> Next.Record\n\n'
-            'EOF\n')
-    t = textfsm.TextFSM(StringIO(tplt))
+    tplt = (
+        'Value Required boo (one)\nValue Filldown hoo (two)\n\n'
+        'Start\n  ^$boo -> Next.Record\n  ^$hoo -> Next.Record\n\n'
+        'EOF\n'
+    )
+    t = textfsm.TextFSM(io.StringIO(tplt))
 
     # Matching two lines. Only one records returned due to 'Required' flag.
     # Tests 'Filldown' and 'Required' options.
@@ -440,7 +498,7 @@ class UnitTestFSM(unittest.TestCase):
     result = t.ParseText(data)
     self.assertListEqual(result, [['one', 'two']])
 
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
     # Matching two lines. Two records returned due to 'Filldown' flag.
     data = 'two\none\none'
     t.Reset()
@@ -448,11 +506,13 @@ class UnitTestFSM(unittest.TestCase):
     self.assertListEqual(result, [['one', 'two'], ['one', 'two']])
 
     # Multiple Variables and options.
-    tplt = ('Value Required,Filldown boo (one)\n'
-            'Value Filldown,Required hoo (two)\n\n'
-            'Start\n  ^$boo -> Next.Record\n  ^$hoo -> Next.Record\n\n'
-            'EOF\n')
-    t = textfsm.TextFSM(StringIO(tplt))
+    tplt = (
+        'Value Required,Filldown boo (one)\n'
+        'Value Filldown,Required hoo (two)\n\n'
+        'Start\n  ^$boo -> Next.Record\n  ^$hoo -> Next.Record\n\n'
+        'EOF\n'
+    )
+    t = textfsm.TextFSM(io.StringIO(tplt))
     data = 'two\none\none'
     result = t.ParseText(data)
     self.assertListEqual(result, [['one', 'two'], ['one', 'two']])
@@ -461,7 +521,7 @@ class UnitTestFSM(unittest.TestCase):
 
     # Trivial FSM, no records produced.
     tplt = 'Value unused (.)\n\nStart\n  ^Trivial SFM\n'
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
 
     data = 'Non-matching text\nline1\nline 2\n'
     self.assertFalse(t.ParseText(data))
@@ -471,7 +531,7 @@ class UnitTestFSM(unittest.TestCase):
 
     # Simple FSM, One Variable no options.
     tplt = 'Value boo (.*)\n\nStart\n  ^$boo -> Next.Record\n\nEOF\n'
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
 
     # Matching one line.
     # Tests 'Next' & 'Record' actions.
@@ -483,14 +543,17 @@ class UnitTestFSM(unittest.TestCase):
     t.Reset()
     data = 'Matching text\nAnd again'
     result = t.ParseTextToDicts(data)
-    self.assertListEqual(result,
-                     [{'boo': 'Matching text'}, {'boo': 'And again'}])
+    self.assertListEqual(
+        result, [{'boo': 'Matching text'}, {'boo': 'And again'}]
+    )
 
     # Two Variables and singular options.
-    tplt = ('Value Required boo (one)\nValue Filldown hoo (two)\n\n'
-            'Start\n  ^$boo -> Next.Record\n  ^$hoo -> Next.Record\n\n'
-            'EOF\n')
-    t = textfsm.TextFSM(StringIO(tplt))
+    tplt = (
+        'Value Required boo (one)\nValue Filldown hoo (two)\n\n'
+        'Start\n  ^$boo -> Next.Record\n  ^$hoo -> Next.Record\n\n'
+        'EOF\n'
+    )
+    t = textfsm.TextFSM(io.StringIO(tplt))
 
     # Matching two lines. Only one records returned due to 'Required' flag.
     # Tests 'Filldown' and 'Required' options.
@@ -498,30 +561,34 @@ class UnitTestFSM(unittest.TestCase):
     result = t.ParseTextToDicts(data)
     self.assertListEqual(result, [{'hoo': 'two', 'boo': 'one'}])
 
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
     # Matching two lines. Two records returned due to 'Filldown' flag.
     data = 'two\none\none'
     t.Reset()
     result = t.ParseTextToDicts(data)
     self.assertListEqual(
-        result, [{'hoo': 'two', 'boo': 'one'}, {'hoo': 'two', 'boo': 'one'}])
+        result, [{'hoo': 'two', 'boo': 'one'}, {'hoo': 'two', 'boo': 'one'}]
+    )
 
     # Multiple Variables and options.
-    tplt = ('Value Required,Filldown boo (one)\n'
-            'Value Filldown,Required hoo (two)\n\n'
-            'Start\n  ^$boo -> Next.Record\n  ^$hoo -> Next.Record\n\n'
-            'EOF\n')
-    t = textfsm.TextFSM(StringIO(tplt))
+    tplt = (
+        'Value Required,Filldown boo (one)\n'
+        'Value Filldown,Required hoo (two)\n\n'
+        'Start\n  ^$boo -> Next.Record\n  ^$hoo -> Next.Record\n\n'
+        'EOF\n'
+    )
+    t = textfsm.TextFSM(io.StringIO(tplt))
     data = 'two\none\none'
     result = t.ParseTextToDicts(data)
     self.assertListEqual(
-        result, [{'hoo': 'two', 'boo': 'one'}, {'hoo': 'two', 'boo': 'one'}])
+        result, [{'hoo': 'two', 'boo': 'one'}, {'hoo': 'two', 'boo': 'one'}]
+    )
 
   def testParseNullText(self):
 
     # Simple FSM, One Variable no options.
     tplt = 'Value boo (.*)\n\nStart\n  ^$boo -> Next.Record\n\n'
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
 
     # Null string
     data = ''
@@ -531,181 +598,210 @@ class UnitTestFSM(unittest.TestCase):
   def testReset(self):
 
     tplt = 'Value boo (.*)\n\nStart\n  ^$boo -> Next.Record\n\nEOF\n'
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
     data = 'Matching text'
     result1 = t.ParseText(data)
     t.Reset()
     result2 = t.ParseText(data)
     self.assertListEqual(result1, result2)
 
-    tplt = ('Value boo (one)\nValue hoo (two)\n\n'
-            'Start\n  ^$boo -> State1\n\n'
-            'State1\n  ^$hoo -> Start\n\n'
-            'EOF')
-    t = textfsm.TextFSM(StringIO(tplt))
+    tplt = (
+        'Value boo (one)\nValue hoo (two)\n\n'
+        'Start\n  ^$boo -> State1\n\n'
+        'State1\n  ^$hoo -> Start\n\n'
+        'EOF'
+    )
+    t = textfsm.TextFSM(io.StringIO(tplt))
 
     data = 'one'
     t.ParseText(data)
     t.Reset()
     self.assertEqual(t._cur_state[0].match, '^$boo')
-    self.assertEqual(t._GetValue('boo').value, None)
-    self.assertEqual(t._GetValue('hoo').value, None)
+    self.assertIsNone(None, t._GetValue('boo').value)
+    self.assertIsNone(t._GetValue('hoo').value)
     self.assertEqual(t._result, [])
 
   def testClear(self):
 
     # Clear Filldown variable.
     # Tests 'Clear'.
-    tplt = ('Value Required boo (on.)\n'
-            'Value Filldown,Required hoo (tw.)\n\n'
-            'Start\n  ^$boo -> Next.Record\n  ^$hoo -> Next.Clear')
+    tplt = (
+        'Value Required boo (on.)\n'
+        'Value Filldown,Required hoo (tw.)\n\n'
+        'Start\n  ^$boo -> Next.Record\n  ^$hoo -> Next.Clear'
+    )
 
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
     data = 'one\ntwo\nonE\ntwO'
     result = t.ParseText(data)
     self.assertListEqual(result, [['onE', 'two']])
 
     # Clearall, with Filldown variable.
     # Tests 'Clearall'.
-    tplt = ('Value Filldown boo (on.)\n'
-            'Value Filldown hoo (tw.)\n\n'
-            'Start\n  ^$boo -> Next.Clearall\n'
-            '  ^$hoo')
+    tplt = (
+        'Value Filldown boo (on.)\n'
+        'Value Filldown hoo (tw.)\n\n'
+        'Start\n  ^$boo -> Next.Clearall\n'
+        '  ^$hoo'
+    )
 
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
     data = 'one\ntwo'
     result = t.ParseText(data)
     self.assertListEqual(result, [['', 'two']])
 
   def testContinue(self):
 
-    tplt = ('Value Required boo (on.)\n'
-            'Value Filldown,Required hoo (on.)\n\n'
-            'Start\n  ^$boo -> Continue\n  ^$hoo -> Continue.Record')
+    tplt = (
+        'Value Required boo (on.)\n'
+        'Value Filldown,Required hoo (on.)\n\n'
+        'Start\n  ^$boo -> Continue\n  ^$hoo -> Continue.Record'
+    )
 
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
     data = 'one\non0'
     result = t.ParseText(data)
     self.assertListEqual(result, [['one', 'one'], ['on0', 'on0']])
 
   def testError(self):
 
-    tplt = ('Value Required boo (on.)\n'
-            'Value Filldown,Required hoo (on.)\n\n'
-            'Start\n  ^$boo -> Continue\n  ^$hoo -> Error')
+    tplt = (
+        'Value Required boo (on.)\n'
+        'Value Filldown,Required hoo (on.)\n\n'
+        'Start\n  ^$boo -> Continue\n  ^$hoo -> Error'
+    )
 
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
     data = 'one'
     self.assertRaises(textfsm.TextFSMError, t.ParseText, data)
 
-    tplt = ('Value Required boo (on.)\n'
-            'Value Filldown,Required hoo (on.)\n\n'
-            'Start\n  ^$boo -> Continue\n  ^$hoo -> Error "Hello World"')
+    tplt = (
+        'Value Required boo (on.)\n'
+        'Value Filldown,Required hoo (on.)\n\n'
+        'Start\n  ^$boo -> Continue\n  ^$hoo -> Error "Hello World"'
+    )
 
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
     self.assertRaises(textfsm.TextFSMError, t.ParseText, data)
 
   def testKey(self):
-    tplt = ('Value Required boo (on.)\n'
-            'Value Required,Key hoo (on.)\n\n'
-            'Start\n  ^$boo -> Continue\n  ^$hoo -> Record')
+    tplt = (
+        'Value Required boo (on.)\n'
+        'Value Required,Key hoo (on.)\n\n'
+        'Start\n  ^$boo -> Continue\n  ^$hoo -> Record'
+    )
 
-    t = textfsm.TextFSM(StringIO(tplt))
-    self.assertTrue('Key' in t._GetValue('hoo').OptionNames())
-    self.assertTrue('Key' not in t._GetValue('boo').OptionNames())
+    t = textfsm.TextFSM(io.StringIO(tplt))
+    self.assertIn('Key', t._GetValue('hoo').OptionNames())
+    self.assertNotIn('Key', t._GetValue('boo').OptionNames())
 
   def testList(self):
 
-    tplt = ('Value List boo (on.)\n'
-            'Value hoo (tw.)\n\n'
-            'Start\n  ^$boo\n  ^$hoo -> Next.Record\n\n'
-            'EOF')
+    tplt = (
+        'Value List boo (on.)\n'
+        'Value hoo (tw.)\n\n'
+        'Start\n  ^$boo\n  ^$hoo -> Next.Record\n\n'
+        'EOF'
+    )
 
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
     data = 'one\ntwo\non0\ntw0'
     result = t.ParseText(data)
     self.assertListEqual(result, [[['one'], 'two'], [['on0'], 'tw0']])
 
-    tplt = ('Value List,Filldown boo (on.)\n'
-            'Value hoo (on.)\n\n'
-            'Start\n  ^$boo -> Continue\n  ^$hoo -> Next.Record\n\n'
-            'EOF')
+    tplt = (
+        'Value List,Filldown boo (on.)\n'
+        'Value hoo (on.)\n\n'
+        'Start\n  ^$boo -> Continue\n  ^$hoo -> Next.Record\n\n'
+        'EOF'
+    )
 
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
     data = 'one\non0\non1'
     result = t.ParseText(data)
-    self.assertEqual(result, ([[['one'], 'one'],
-                               [['one', 'on0'], 'on0'],
-                               [['one', 'on0', 'on1'], 'on1']]))
+    self.assertEqual(
+        result,
+        ([
+            [['one'], 'one'],
+            [['one', 'on0'], 'on0'],
+            [['one', 'on0', 'on1'], 'on1'],
+        ]),
+    )
 
-    tplt = ('Value List,Required boo (on.)\n'
-            'Value hoo (tw.)\n\n'
-            'Start\n  ^$boo -> Continue\n  ^$hoo -> Next.Record\n\n'
-            'EOF')
+    tplt = (
+        'Value List,Required boo (on.)\n'
+        'Value hoo (tw.)\n\n'
+        'Start\n  ^$boo -> Continue\n  ^$hoo -> Next.Record\n\n'
+        'EOF'
+    )
 
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
     data = 'one\ntwo\ntw2'
     result = t.ParseText(data)
     self.assertListEqual(result, [[['one'], 'two']])
 
-
   def testNestedMatching(self):
-      """
-      Ensures that List-type values with nested regex capture groups are parsed
-      correctly as a list of dictionaries.
+    """List-type values with nested regex capture groups are parsed correctly.
 
-      Additionaly, another value is used with the same group-name as one of the
-      nested groups to ensure that there are no conflicts when the same name is
-      used.
-      """
-      tplt = (
-          # A nested group is called "name"
-          r"Value List foo ((?P<name>\w+):\s+(?P<age>\d+)\s+(?P<state>\w{2})\s*)"
-          "\n"
-          # A regular value is called "name"
-          r"Value name (\w+)"
-          # "${name}" here refers to the Value called "name"
-          "\n\nStart\n"
-          r"  ^\s*${foo}"
-          "\n"
-          r"  ^\s*${name}"
-          "\n"
-          r"  ^\s*$$ -> Record"
-      )
-      t = textfsm.TextFSM(StringIO(tplt))
-      # Julia should be parsed as "name" separately
-      data = " Bob: 32 NC\n Alice: 27 NY\n Jeff: 45 CA\nJulia\n\n"
-      result = t.ParseText(data)
-      self.assertListEqual(
-          result, (
-              [[[
-                  {'name': 'Bob', 'age': '32', 'state': 'NC'},
-                  {'name': 'Alice', 'age': '27', 'state': 'NY'},
-                  {'name': 'Jeff', 'age': '45', 'state': 'CA'}
-              ], 'Julia']]
-          )
-      )
+    Additionaly, another value is used with the same group-name as one of the
+    nested groups to ensure that there are no conflicts when the same name is
+    used.
+    """
+
+    tplt = (
+        # A nested group is called "name"
+        r'Value List foo ((?P<name>\w+):\s+(?P<age>\d+)\s+(?P<state>\w{2})\s*)'
+        '\n'
+        # A regular value is called "name"
+        r'Value name (\w+)'
+        # "${name}" here refers to the Value called "name"
+        '\n\nStart\n'
+        r'  ^\s*${foo}'
+        '\n'
+        r'  ^\s*${name}'
+        '\n'
+        r'  ^\s*$$ -> Record'
+    )
+    t = textfsm.TextFSM(io.StringIO(tplt))
+    # Julia should be parsed as "name" separately
+    data = ' Bob: 32 NC\n Alice: 27 NY\n Jeff: 45 CA\nJulia\n\n'
+    result = t.ParseText(data)
+    self.assertListEqual(
+        result,
+        ([[
+            [
+                {'name': 'Bob', 'age': '32', 'state': 'NC'},
+                {'name': 'Alice', 'age': '27', 'state': 'NY'},
+                {'name': 'Jeff', 'age': '45', 'state': 'CA'},
+            ],
+            'Julia',
+        ]]),
+    )
 
   def testNestedNameConflict(self):
-      tplt = (
-          # Two nested groups are called "name"
-          r"Value List foo ((?P<name>\w+)\s+(?P<name>\w+):\s+(?P<age>\d+)\s+(?P<state>\w{2})\s*)"
-          "\nStart\n"
-          r"^\s*${foo}"
-          "\n  ^"
-          r"\s*$$ -> Record"
-      )
-      self.assertRaises(textfsm.TextFSMTemplateError, textfsm.TextFSM, StringIO(tplt))
-
+    tplt = (
+        # Two nested groups are called "name"
+        r'Value List foo'
+        r' ((?P<name>\w+)\s+(?P<name>\w+):\s+(?P<age>\d+)\s+(?P<state>\w{2})\s*)'
+        '\nStart\n'
+        r'^\s*${foo}'
+        '\n  ^'
+        r'\s*$$ -> Record'
+    )
+    self.assertRaises(
+        textfsm.TextFSMTemplateError, textfsm.TextFSM, io.StringIO(tplt)
+    )
 
   def testGetValuesByAttrib(self):
 
-    tplt = ('Value Required boo (on.)\n'
-            'Value Required,List hoo (on.)\n\n'
-            'Start\n  ^$boo -> Continue\n  ^$hoo -> Record')
+    tplt = (
+        'Value Required boo (on.)\n'
+        'Value Required,List hoo (on.)\n\n'
+        'Start\n  ^$boo -> Continue\n  ^$hoo -> Record'
+    )
 
     # Explicit default.
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
     self.assertEqual(t.GetValuesByAttrib('List'), ['hoo'])
     self.assertEqual(t.GetValuesByAttrib('Filldown'), [])
     result = t.GetValuesByAttrib('Required')
@@ -715,37 +811,41 @@ class UnitTestFSM(unittest.TestCase):
   def testStateChange(self):
 
     # Sinple state change, no actions
-    tplt = ('Value boo (one)\nValue hoo (two)\n\n'
-            'Start\n  ^$boo -> State1\n\nState1\n  ^$hoo -> Start\n\n'
-            'EOF')
-    t = textfsm.TextFSM(StringIO(tplt))
+    tplt = (
+        'Value boo (one)\nValue hoo (two)\n\n'
+        'Start\n  ^$boo -> State1\n\nState1\n  ^$hoo -> Start\n\n'
+        'EOF'
+    )
+    t = textfsm.TextFSM(io.StringIO(tplt))
 
     data = 'one'
     t.ParseText(data)
     self.assertEqual(t._cur_state[0].match, '^$hoo')
     self.assertEqual('one', t._GetValue('boo').value)
-    self.assertEqual(None, t._GetValue('hoo').value)
+    self.assertIsNone(t._GetValue('hoo').value)
     self.assertEqual(t._result, [])
 
     # State change with actions.
-    tplt = ('Value boo (one)\nValue hoo (two)\n\n'
-            'Start\n  ^$boo -> Next.Record State1\n\n'
-            'State1\n  ^$hoo -> Start\n\n'
-            'EOF')
-    t = textfsm.TextFSM(StringIO(tplt))
+    tplt = (
+        'Value boo (one)\nValue hoo (two)\n\n'
+        'Start\n  ^$boo -> Next.Record State1\n\n'
+        'State1\n  ^$hoo -> Start\n\n'
+        'EOF'
+    )
+    t = textfsm.TextFSM(io.StringIO(tplt))
 
     data = 'one'
     t.ParseText(data)
     self.assertEqual(t._cur_state[0].match, '^$hoo')
-    self.assertEqual(None, t._GetValue('boo').value)
-    self.assertEqual(None, t._GetValue('hoo').value)
+    self.assertIsNone(t._GetValue('boo').value)
+    self.assertIsNone(t._GetValue('hoo').value)
     self.assertEqual(t._result, [['one', '']])
 
   def testEOF(self):
 
     # Implicit EOF.
     tplt = 'Value boo (.*)\n\nStart\n  ^$boo -> Next\n'
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
 
     data = 'Matching text'
     result = t.ParseText(data)
@@ -753,14 +853,14 @@ class UnitTestFSM(unittest.TestCase):
 
     # EOF explicitly suppressed in template.
     tplt = 'Value boo (.*)\n\nStart\n  ^$boo -> Next\n\nEOF\n'
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
 
     result = t.ParseText(data)
     self.assertListEqual(result, [])
 
     # Implicit EOF suppressed by argument.
     tplt = 'Value boo (.*)\n\nStart\n  ^$boo -> Next\n'
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
 
     result = t.ParseText(data, eof=False)
     self.assertListEqual(result, [])
@@ -769,7 +869,7 @@ class UnitTestFSM(unittest.TestCase):
 
     # End State, EOF is skipped.
     tplt = 'Value boo (.*)\n\nStart\n  ^$boo -> End\n  ^$boo -> Record\n'
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
     data = 'Matching text A\nMatching text B'
 
     result = t.ParseText(data)
@@ -777,14 +877,14 @@ class UnitTestFSM(unittest.TestCase):
 
     # End State, with explicit Record.
     tplt = 'Value boo (.*)\n\nStart\n  ^$boo -> Record End\n'
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
 
     result = t.ParseText(data)
     self.assertListEqual(result, [['Matching text A']])
 
     # EOF state transition is followed by implicit End State.
     tplt = 'Value boo (.*)\n\nStart\n  ^$boo -> EOF\n  ^$boo -> Record\n'
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
 
     result = t.ParseText(data)
     self.assertListEqual(result, [['Matching text A']])
@@ -792,14 +892,15 @@ class UnitTestFSM(unittest.TestCase):
   def testInvalidRegexp(self):
 
     tplt = 'Value boo (.$*)\n\nStart\n  ^$boo -> Next\n'
-    self.assertRaises(textfsm.TextFSMTemplateError,
-                      textfsm.TextFSM, StringIO(tplt))
+    self.assertRaises(
+        textfsm.TextFSMTemplateError, textfsm.TextFSM, io.StringIO(tplt)
+    )
 
   def testValidRegexp(self):
     """RegexObjects uncopyable in Python 2.6."""
 
     tplt = 'Value boo (fo*)\n\nStart\n  ^$boo -> Record\n'
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
     data = 'f\nfo\nfoo\n'
     result = t.ParseText(data)
     self.assertListEqual(result, [['f'], ['fo'], ['foo']])
@@ -809,7 +910,7 @@ class UnitTestFSM(unittest.TestCase):
 
     tplt = 'Value boo (.*)\n\nStart\n  ^$boo -> Next Stop\n\nStop\n  ^abc\n'
     output_text = 'one\ntwo'
-    tmpl_file = StringIO(tplt)
+    tmpl_file = io.StringIO(tplt)
 
     t = textfsm.TextFSM(tmpl_file)
     t.ParseText(output_text)
@@ -833,10 +934,11 @@ Start
 2 A2 --
 3 -- B3
 """
-    t = textfsm.TextFSM(StringIO(tplt))
+    t = textfsm.TextFSM(io.StringIO(tplt))
     result = t.ParseText(data)
     self.assertListEqual(
-        result, [['1', 'A2', 'B1'], ['2', 'A2', 'B3'], ['3', '', 'B3']])
+        result, [['1', 'A2', 'B1'], ['2', 'A2', 'B3'], ['3', '', 'B3']]
+    )
 
 
 class UnitTestUnicode(unittest.TestCase):
@@ -895,7 +997,7 @@ State1
   ^$$ -> Next
   ^$$ -> End
 """
-    f = StringIO(buf)
+    f = io.StringIO(buf)
     t = textfsm.TextFSM(f)
     self.assertEqual(str(t), buf_result)
 
